
/*****************************************************************************
**
** File:        mqwrapget.c
** Purpose:     Natural Interface for MQSeries
** Description: MQGET function
**
** (c) Copyright 2000 by IBM
**
*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>             /* Memory and type conversions */
#include <string.h>             /* strcpy stuff */
#include <ctype.h>              /* is upper stuff */
#include <time.h>               /* time of course */
#include <errno.h>              /* contains errno , last error indicator */
#include <fcntl.h>              /* FILE I/O related protos and defines */
#include <limits.h>		/* FILE I/O related protos and defines */
#include <io.h>             /* defines for file access              */
#include <sys/types.h>          /* for data types sycj as time          */
#include <sys/stat.h>           /* finding out status info              */

#include "cmqc.h"		        /* MQ Headers		*/
#include "natuser.h"	    	/* Natural User Exit	*/
#include "namq.h"               /* MQ interface function prototypes */

static FILE *dbgfile = NULL;

NATFCT NAMQGET NATARGDEF(nparm, parmptr, parmdec)
{
#ifdef MQGET_DEBUG
  static char * func = "NAMQGET : "	;
#endif

  MQHCONN	con_hndl	;
  MQHOBJ        obj_hndl        ;
  MQMD          msg_desc = {MQMD_DEFAULT}       ;
  MQGMO         get_options = {MQGMO_DEFAULT}   ;
  MQLONG 	buf_length 	;
  MQBYTE*       buffer          ;
  MQLONG        data_length     ;
  MQLONG 	cc		;
  MQLONG 	rc 		;

#ifdef MQGET_DEBUG
   dbgfile = dbgopen();
#endif

  /*
  ** test number of arguments
  */
  if (nparm != 9 )
       {
#ifdef MQGET_DEBUG
   		if (dbgfile != NULL) {
    		fprintf(dbgfile, "\n%s Wrong number of Parameters!, got '%d'\n" ,func,
                                        nparm) ;
			dbgclose();
	 	}
#endif

		return 1;
  	 }

  /*
  ** move Con Handle from Natural to Local variable
  */

  memmove ((char *) &con_hndl, (char *) parmptr[0], sizeof(MQHCONN));

#ifdef MQGET_DEBUG
  if (dbgfile != NULL) {
  	fprintf(dbgfile, "\n\n%s con handle: %d\n", func , con_hndl ) ;
  }
#endif

  /*
  ** move Object Handle from Natural to Local variable
  */

  memmove ((char *) &obj_hndl, (char *) parmptr[1], sizeof(MQHOBJ));

#ifdef MQGET_DEBUG
  if (dbgfile != NULL) {
  	fprintf(dbgfile, "\n%s object handle: %d\n", func , obj_hndl ) ;
  }
#endif

  /*
  ** move MessageDescriptor from Natural to Local variable
  */

  memmove ((char *) &msg_desc, (char *) parmptr[2], sizeof(MQMD));

  /*
  ** move Get Message Options from Natural to Local variable
  */

  memmove ((char *) &get_options, (char *) parmptr[3], sizeof(MQGMO));

  /*
  ** move Buffer length from Natural to Local variable
  */

  memmove ((char *) &buf_length, (char *) parmptr[4], sizeof(MQLONG));

#ifdef MQGET_DEBUG
  if (dbgfile != NULL) {
	 fprintf(dbgfile, "\n%s buffer length: %d\n", func , buf_length ) ;
  }
#endif

  /*
  ** poit local variable buffer to Natural structure
  */
  buffer = (char *) parmptr[5];

  /*
  ** Call the get routine
  */

  mq_get ( con_hndl, obj_hndl, &msg_desc, &get_options, buf_length,
                buffer, &data_length, &cc, &rc);

#ifdef MQGET_DEBUG
  if (dbgfile != NULL) {
    fprintf(dbgfile, "\n%s Get Result: cc = %d , rc = %d \n", func , cc , rc ) ;

      dbgclose() ;
  }
#endif

  /*
  ** move buffer to Natural Return block
  */

  if (data_length > 0) {
	  memmove ((char *) parmptr[5], (char *) buffer, data_length);

#ifdef MQGET_DEBUG
     if (dbgfile != NULL) {
	   char *bdata = malloc(data_length+1);
	   dbgtrunc(bdata, buffer, data_length, 0);
       fprintf(dbgfile, "\nmsg get : buffer '%s'",bdata);
       free(bdata);
     }
#endif

  }

  /*
  ** move datalength to Natural Return block
  */

  memmove ((char *) parmptr[6], (char *) &data_length, sizeof(MQLONG));

  /*
  ** move Condition Code to Natural Return block
  */

  memmove ((char *) parmptr[7], (char *) &cc, sizeof(MQLONG));

  /*
  ** move Return Code to Natural Return block
  */

  memmove ((char *) parmptr[8], (char *) &rc, sizeof(MQLONG));

  #ifdef MQGET_DEBUG
    if (dbgfile != NULL) {
      dbgclose() ;
    }
#endif

  return 0;

} /* MQGET */


/*
***********************************************************************
			mq_get
***********************************************************************
*/
void mq_get (MQHCONN con_hndl, MQHOBJ obj_hndl, MQMD * msg_desc,
               MQGMO * get_options, MQLONG buf_length, MQBYTE * buffer,
               MQLONG * data_length, MQLONG *cc, MQLONG *rc)
{
#ifdef MQGET_DEBUG
  static char * func = "mq_get : "	;
#endif

    MQGET(con_hndl, obj_hndl, msg_desc, get_options, buf_length,
                        buffer, data_length, cc , rc) ;

#ifdef MQGET_DEBUG
    if ( *cc == MQCC_FAILED) {
  		if (dbgfile != NULL) {
	   		fprintf(dbgfile, "%s Get Failed, rc = %d\n",func, *rc);
		}
	}
#endif

    return ;

}


