
/*****************************************************************************
**
** File:        mqwrapput1.c
** Purpose:     Natural Interface for MQSeries
** Description: MQPUT1 function
**
** (c) Copyright 2000 by IBM
**
*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>             /* Memory and type conversions */
#include <string.h>             /* strcpy stuff */
#include <ctype.h>              /* is upper stuff */
#include <time.h>               /* time of course */
#include <errno.h>              /* contains errno , last error indicator */
#include <fcntl.h>              /* FILE I/O related protos and defines */
#include <limits.h>		/* FILE I/O related protos and defines */
#include <io.h>             /* defines for file access              */
#include <sys/types.h>          /* for data types sycj as time          */
#include <sys/stat.h>           /* finding out status info              */

#include "cmqc.h"		        /* MQ Headers		*/
#include "natuser.h"	    	/* Natural User Exit	*/
#include "namq.h"               /* MQ interface function prototypes */

NATFCT NAMQPUT1 NATARGDEF(nparm, parmptr, parmdec)
{
#ifdef MQPUT_DEBUG
  static char * func = "NAMQPUT1 : "	;
#endif

  MQHCONN	con_hndl	;
  MQOD          obj_desc = {MQOD_DEFAULT}       ;
  MQMD          msg_desc = {MQMD_DEFAULT}       ;
  MQPMO         put_options = {MQPMO_DEFAULT}   ;
  MQLONG 	buf_length 	;
  MQBYTE*       buffer          ;
  MQLONG 	cc		;
  MQLONG 	rc 		;

  /*
  ** test number of arguments
  */
  if (nparm != 8 )
       {
#ifdef MQPUT_DEBUG
    		printf("\n%s Wrong number of Parameters! nr is '%d'" ,func,
                                                        nparm) ;
#endif
	      	return 1;
  	 }

  /*
  ** move Con Handle from Natural to Local variable
  */

  memmove ((char *) &con_hndl, (char *) parmptr[0], sizeof(MQHCONN));

#ifdef MQPUT_DEBUG
  printf("\n%s con handle is '%d'", func , con_hndl ) ;
#endif

  /*
  ** move Object Descriptor from Natural to Local variable
  */

  memmove ((char *) &obj_desc, (char *) parmptr[1], sizeof(MQOD));

#ifdef MQPUT_DEBUG
  printf("\n%s object name is '%s'", func , obj_desc.ObjectName ) ;
#endif

  /*
  ** move Message Descriptor from Natural to Local variable
  */

  memmove ((char *) &msg_desc, (char *) parmptr[2], sizeof(MQMD));
#ifdef MQPUT_DEBUG
  printf("\n%s msg desc strucid is '%s'", func , msg_desc.StrucId ) ;
  printf("\n%s msg desc version is '%d'", func , msg_desc.Version ) ;
  printf("\n%s msg desc priority is '%d'", func , msg_desc.Priority ) ;
  printf("\n%s msg desc orig length is '%d'", func , msg_desc.OriginalLength) ;
#endif

  /*
  ** move PutOptions from Natural to Local variable
  */

  memmove ((char *) &put_options, (char *) parmptr[3], sizeof(MQPMO));
  if (put_options.RecsPresent > 0)
  {
#ifdef MQPUT_DEBUG
    printf("\n%s no support for distribution list, RecsPresent = '%d'",
                        func, put_options.RecsPresent);
#endif
    return 2;
    }

#ifdef MQPUT_DEBUG
  printf("\n%s Pmo strucid is '%s'", func , put_options.StrucId ) ;
  printf("\n%s Pmo version is '%d'", func , put_options.Version ) ;
  printf("\n%s Put Options Options is '%d'", func , put_options.Options ) ;
  printf("\n%s Put Options Invalid dest count is '%d'", func ,
                put_options.InvalidDestCount ) ;
  printf("\n%s Put Options recpresent is '%d'", func ,
                put_options.RecsPresent ) ;
  printf("\n%s Put Options resp offset is '%d'", func ,
                                put_options.ResponseRecOffset ) ;
  printf("\n%s Put message options size is '%d'", func, sizeof(MQPMO));
#endif

  /*
  ** move Buffer length from Natural to Local variable
  */

  memmove ((char *) &buf_length, (char *) parmptr[4], sizeof(MQLONG));

#ifdef MQPUT_DEBUG
  printf("\n%s buffer length is '%d'", func , buf_length ) ;
#endif

  /*
  ** Point local variable Buffer to Natural structure
  */
  buffer = (char *) parmptr[5];

#ifdef MQPUT_DEBUG
  printf("\n%s buffer is '%s'", func , buffer) ;
#endif

  /*
  ** Call the put routine
  */
  mq_put1 ( con_hndl, &obj_desc, &msg_desc, &put_options, buf_length,
                buffer, &cc, &rc);

#ifdef MQPUT_DEBUG
  printf("\n%s Put1 Result: cc is %d , rc = %d ", func , cc , rc ) ;
#endif

  /*
  ** move Message Descriptor to Natural Return block
  */
  memmove ((char *) parmptr[2], (char *) &msg_desc, sizeof(MQMD));

  /*
  ** move Put options to Natural Return block
  */
  memmove ((char *) &put_options, (char *) parmptr[3], sizeof(MQPMO));

  /*
  ** move Condition Code to Natural Return block
  */

  memmove ((char *) parmptr[6], (char *) &cc, sizeof(MQLONG));

  /*
  ** move Return Code to Natural Return block
  */

  memmove ((char *) parmptr[7], (char *) &rc, sizeof(MQLONG));

  return 0;

} /* MQPUT1 */


/*
***********************************************************************
			mq_put1
***********************************************************************
*/
void mq_put1 (MQHCONN con_hndl, MQOD * obj_desc, MQMD * msg_desc,
               MQPMO * put_options, MQLONG buf_length, MQBYTE * buffer,
               MQLONG *cc, MQLONG *rc)
{
#ifdef MQPUT_DEBUG
  static char * func = "mq_put1 : "	;
#endif

    MQPUT1(con_hndl, obj_desc, msg_desc, put_options, buf_length,
                        buffer, cc , rc) ;

#ifdef MQPUT_DEBUG
    if ( *cc == MQCC_FAILED)
	{
	   printf("%s Failed to Put %d",func, *rc);
	}
#endif

    return ;

}


